/*
    This file is part of matchbox-tray-gsm

    Matchbox-tray-gsm is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Matchbox-tray-gsm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with matchbox-tray-gsm.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
Stuff to wrap these unfamiliar framework-using-dbus-using-glib-calls in nice plain ole C calls.

Yes, I know I oughtta love Glib but atm I just can't get used to it, so I'll wrap it in standard C wherever
possible. So there.
*/

#include "marshal.h"
#include "dbus-gsm.h"
#include <stdlib.h>

DBusGConnection *connection;
DBusGProxy *proxy;

//Create a proxy for the GSM.Network-interface.
DBusGProxy* DbusCreateProxy() {
    GError *error;

    g_type_init();

    error=NULL;
    //Create a system-level dbus connection
    connection=dbus_g_bus_get(DBUS_BUS_SYSTEM, &error);
  
    if (connection==NULL) {
	g_printerr ("Failed to open connection to bus: %s\n", error->message);
	g_error_free (error);
	exit (1);
    }

    /* Create a proxy object for the "bus driver" */
    //conn name path iface
    proxy = dbus_g_proxy_new_for_name(connection, \
	"org.freesmartphone.ogsmd", "/org/freesmartphone/GSM/Device", "org.freesmartphone.GSM.Network");
    return proxy;
}

void GSMSetSignalCallback(void(*cb)(DBusGProxy*,int,gpointer)) {
    //Set callback for the 'SignalStrength'-signal
    dbus_g_object_register_marshaller(marshal_VOID__INT, G_TYPE_NONE, G_TYPE_INT, G_TYPE_INVALID);
    dbus_g_proxy_add_signal(proxy,"SignalStrength",G_TYPE_INT,G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(proxy,"SignalStrength",G_CALLBACK(cb),connection,NULL);
}

void DBusFreeProxy(DBusGProxy *proxy) {
    g_object_unref(proxy);
}

//Get the signal strength from the modem.
int GSMGetSignalStrengthPct(DBusGProxy *proxy) {
    int str;
    GError *error;
    int strength;

    error = NULL;
    if (!dbus_g_proxy_call (proxy, "GetSignalStrength", &error, G_TYPE_INVALID, G_TYPE_INT, &strength, G_TYPE_INVALID)) {
	if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION) {
	    g_printerr ("Caught remote method exception %s: %s", dbus_g_error_get_name (error), error->message);
	} else {
	    g_printerr ("Error: %s\n", error->message);
	}
	g_error_free (error);
	exit (1);
    }

    return strength;
}
