/*
    This file is part of matchbox-tray-gsm

    Matchbox-tray-gsm is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Matchbox-tray-gsm is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with matchbox-tray-gsm.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <libmb/mb.h>
#include "dbus-gsm.h"
#include <pthread.h>
#include <stdio.h>

#define GFX_DIR "/usr/local/lib/matchbox-tray-gsm/gfx"

//Holds the tray app icons
MBPixbufImage *AppImage[6];
//Proxy to the gsm.network-interface
DBusGProxy* proxy;
//Holds the GSM strength in pct.
int gsmstrength;
//The app handle.
MBTrayApp *app=NULL;
//A pixbuff.
MBPixbuf *Pixbuf;


//Called when the icon needs to be painted, either because of some X event or because
//signal_callback requested it
void paint_callback(MBTrayApp *app, Drawable drw) {
    int signalpic;
    MBPixbufImage *img_scaled;
    MBPixbufImage *img_tray;
    int i;
    //Signal thresholds. This holds in position n how much pct the signal strength should be before
    //the n'th bar appears. This is to logarithmically increase the amount of bars, which is what other
    //phones seem to do. This array isn't calibrated itself though. If you think the signal indicated
    //is too low, just change this array and recompile :)
    //
    const int signalthreshold[]={30,55,75,85,92};
    
    //Find the pic index for the current signal strength.
    signalpic=0;
    for (i=0; i<5; i++) {
	if (gsmstrength>=signalthreshold[i]) signalpic=i+1;
    }

    //Get the background and place the correct signal pic on top of it
    img_tray=mb_tray_app_get_background(app, Pixbuf);
    img_scaled=mb_pixbuf_img_scale(Pixbuf, AppImage[signalpic], mb_tray_app_width(app), mb_tray_app_height(app));
    mb_pixbuf_img_copy_composite(Pixbuf,img_tray,img_scaled,0,0,mb_tray_app_width(app), mb_tray_app_height(app),0,0);
    //render
    mb_pixbuf_img_render_to_drawable(Pixbuf, img_tray, drw, 0, 0);
    mb_pixbuf_img_free(Pixbuf, img_scaled);
}

//Called when the tray icon is clicked
void button_callback(MBTrayApp *app, int x, int y, Bool is_released) {
    char buff[20];
    if (is_released) {
	//For now, just display the signal strength in pct. ToDo: add provider etc. info
	snprintf(buff,20,"%i%%",gsmstrength);
	mb_tray_app_tray_send_message(app, buff, 3000);
    }
}

//Called when the dbus interface gets a 'Signal'-event, which occurs whenever the
//GSM signal strength changes.
void signal_callback(DBusGProxy *proxy, int strength,gpointer userdata) {
    gsmstrength=strength;
    mb_tray_app_repaint(app);
}

//Thread for the matchbox_tray_app main loop.
void *mbthread_main(void *app) {
    mb_tray_app_main((MBTrayApp *)app);
}

//Main function
int main(int argc, char **argv){
    pthread_t mbthread;
    GMainLoop *loop = NULL;
    int i;

    //Create proxy for freesmartphone.org framework.
    proxy=DbusCreateProxy();
    //Set callback for GSM signal change
    GSMSetSignalCallback(signal_callback);

    //Create glib main loop, for reception of dbus signals.
    loop = g_main_loop_new (NULL, FALSE);

    //Create MatchBox_tray_app stuff.
    app=mb_tray_app_new( "GSM", NULL, paint_callback, &argc,  &argv);
    Pixbuf=mb_pixbuf_new(mb_tray_app_xdisplay(app), mb_tray_app_xscreen(app));
    //Load 6 images with signal strength indicator
    AppImage[0]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/0pct.png");
    AppImage[1]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/20pct.png");
    AppImage[2]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/40pct.png");
    AppImage[3]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/60pct.png");
    AppImage[4]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/80pct.png");
    AppImage[5]=mb_pixbuf_img_new_from_file(Pixbuf, GFX_DIR"/100pct.png");
    for (i=0; i<6; i++) {
	if (AppImage[i]==NULL) {
	    printf("Couldn't load gfx from dir "GFX_DIR". Bailing out.\n");
	    exit(1);
	}
    }
    mb_tray_app_set_button_callback(app, button_callback);

    //Split off matchbox X handling etc to a separate thread, running in parallel
    //with the g_main_loop_run underneath.
    pthread_create(&mbthread,NULL,mbthread_main,(void *)app);

    //Get and draw initial strength
    gsmstrength=GSMGetSignalStrengthPct(proxy);
    mb_tray_app_repaint(app);

    //Set session so it'll get remembered in the panel session
    mb_tray_app_set_session(app,1);

    //Go receive d-bus signals
    g_main_loop_run(loop);
}

